@@ -391,6 +391,13 @@ void FGSubmodelMgr::transform(submodel *sm)
     IC.wind_from_east =  _user_wind_from_east_node->getDoubleValue();
     IC.wind_from_north = _user_wind_from_north_node->getDoubleValue();
 
+    // For submodels affected by wind, convert ground speed to airspeed by subtracting wind.
+    // Since the wind frame is inverted (wind_*from*_{east,north}), the wind values are added.
+    if (sm->wind) {
+        IC.speed_east_fps += IC.wind_from_east;
+        IC.speed_north_fps += IC.wind_from_north;
+    }
+
     userpos.setLatitudeDeg(IC.lat);
     userpos.setLongitudeDeg(IC.lon);
     userpos.setElevationFt(IC.alt);
@@ -484,6 +491,11 @@ void FGSubmodelMgr::setData(int id, const string& path, bool serviceable, const
     SGPropertyNode root;
 
     SGPath config = globals->resolve_aircraft_path(path);
+    if (!config.exists()) {
+        SG_LOG(SG_AI, SG_DEV_ALERT, "missing AI submodels file: " << config);
+        return;
+    }
+
     try {
         SG_LOG(SG_AI, SG_DEBUG,
                 "Submodels: Trying to read AI submodels file: " << config);
@@ -700,9 +712,9 @@ void FGSubmodelMgr::loadSubmodels()
 SGVec3d FGSubmodelMgr::getCartOffsetPos(submodel* sm) const
 {
     // Transform to the right coordinate frame, configuration is done in
-    // the x-forward, y-right, z-up coordinates (feet), computation
-    // in the simulation usual body x-forward, y-right, z-down coordinates
-    // (meters) )
+    // either x-backward, y-right, z-up coordinates (meter),
+    // or (deprecated) x-forward, y-right, z-up coordinates (feet).
+    // computation in the simulation usual body x-forward, y-right, z-down coordinates (meters)
     SGVec3d offset;
     if (sm->offsets_in_meter) {
         offset = SGVec3d(-sm->x_offset->get_value(),
@@ -724,11 +736,6 @@ SGVec3d FGSubmodelMgr::getCartOffsetPos(submodel* sm) const
        IC.elevation,
        IC.roll);
 
-    // postrotate by any pitch/heasing/roll offset
-    hlTrans *= SGQuatd::fromYawPitchRollDeg(sm->yaw_offset->get_value(),
-                                            sm->pitch_offset->get_value(),
-                                            0.0);
-    
     // The offset converted to the usual body fixed coordinate system
     // rotated to the earth-fixed coordinates axis
     SGVec3d off = hlTrans.backTransform(offset);
