@@ -905,10 +905,7 @@ FGTowerController::FGTowerController(FGAirportDynamics *par) :
 
 FGTowerController::~FGTowerController()
 {
-    // to avoid the exception described in:
-    // https://sourceforge.net/p/flightgear/codetickets/1864/
-    // we want to ensure AI aircraft signing-off is a no-op now
-
+    _isDestroying = true;
     clearTrafficControllers(activeTraffic);
 }
 
@@ -1046,6 +1043,10 @@ void FGTowerController::updateAircraftInformation(int id, double lat, double lon
 
 void FGTowerController::signOff(int id)
 {
+    // ensure we don't modify activeTraffic during destruction
+    if (_isDestroying)
+        return;
+
     // Search activeTraffic for a record matching our id
     TrafficVectorIterator i = searchActiveTraffic(activeTraffic, id);
     if (i == activeTraffic.end() || (activeTraffic.empty())) {
@@ -1136,6 +1137,7 @@ FGStartupController::FGStartupController(FGAirportDynamics *par):
 
 FGStartupController::~FGStartupController()
 {
+    _isDestroying = true;
     clearTrafficControllers(activeTraffic);
 }
 
@@ -1208,6 +1210,10 @@ FGATCInstruction FGStartupController::getInstruction(int id)
 
 void FGStartupController::signOff(int id)
 {
+    // ensure we don't modify activeTraffic during destruction
+    if (_isDestroying)
+        return;
+
     // Search activeTraffic for a record matching our id
     TrafficVectorIterator i = searchActiveTraffic(activeTraffic, id);
     
@@ -1264,6 +1270,7 @@ void FGStartupController::updateAircraftInformation(int id, double lat, double l
     if (i == activeTraffic.end() || (activeTraffic.size() == 0)) {
         SG_LOG(SG_ATC, SG_ALERT,
                "AI error: updating aircraft without traffic record at " << SG_ORIGIN);
+        return;
     } else {
         i->setPositionAndHeading(lat, lon, heading, speed, alt);
         current = i;
@@ -1273,7 +1280,8 @@ void FGStartupController::updateAircraftInformation(int id, double lat, double l
     int state = i->getState();
 
     // Sentry FLIGHTGEAR-2Q : don't crash on null TrafficRef
-    if (!i->getAircraft()->getTrafficRef()) {
+    // Sentry FLIGHTGEAR-129: don't crash on null aircraft
+    if (!i->getAircraft() || !i->getAircraft()->getTrafficRef()) {
         SG_LOG(SG_ATC, SG_ALERT, "AI traffic: updating aircraft without traffic ref");
         return;
     }
@@ -1561,6 +1569,7 @@ FGApproachController::FGApproachController(FGAirportDynamics *par):
 
 FGApproachController::~FGApproachController()
 {
+    _isDestroying = true;
     clearTrafficControllers(activeTraffic);
 }
 
@@ -1640,7 +1649,11 @@ void FGApproachController::updateAircraftInformation(int id, double lat, double
 /* Search for and erase traffic record with a specific id */
 void FGApproachController::signOff(int id)
 {
-   // Search activeTraffic for a record matching our id
+    // ensure we don't modify activeTraffic during destruction
+    if (_isDestroying)
+        return;
+
+    // Search activeTraffic for a record matching our id
     TrafficVectorIterator i = searchActiveTraffic(activeTraffic, id);
     
     if (i == activeTraffic.end() || activeTraffic.empty()) {
