@@ -37,12 +37,7 @@ COMMENTS, REFERENCES,  and NOTES
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
-#include <string>
-
 #include "FGFilter.h"
-#include "input_output/FGXMLElement.h"
-#include "input_output/FGPropertyManager.h"
 #include "math/FGParameterValue.h"
 
 using namespace std;
@@ -68,7 +63,7 @@ FGFilter::FGFilter(FGFCS* fcs, Element* element)
 
   CalculateDynamicFilters();
 
-  FGFCSComponent::bind();
+  bind(element);
 
   Debug(0);
 }
@@ -99,8 +94,8 @@ void FGFilter::ReadFilterCoefficients(Element* element, int index)
   coefficient[1] += index;
   
   if ( element->FindElement(coefficient) ) {
-    string property_string = element->FindElementValue(coefficient);
-    C[index] = new FGParameterValue(property_string, PropertyManager);
+    C[index] = new FGParameterValue(element->FindElement(coefficient),
+                                    PropertyManager);
     DynamicFilter |= !C[index]->IsConstant();
   }
 }
@@ -185,7 +180,7 @@ bool FGFilter::Run(void)
   PreviousInput1  = Input;
 
   Clip();
-  if (IsOutput) SetOutput();
+  SetOutput();
 
   return true;
 }
