@@ -7,21 +7,21 @@
  ------------- Copyright (C) 2000 -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -61,10 +61,8 @@ Also, see the header file (FGSwitch.h) for further details.
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
-
 #include "FGSwitch.h"
-#include "input_output/FGXMLElement.h"
+#include "math/FGCondition.h"
 
 using namespace std;
 
@@ -79,8 +77,8 @@ FGSwitch::FGSwitch(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element)
   string value;
   Test *current_test;
 
-  FGFCSComponent::bind(); // Bind() this component here in case it is used
-                          // in its own definition for a sample-and-hold
+  bind(element); // Bind() this component here in case it is used in its own
+                 // definition for a sample-and-hold
   Element* test_element = element->FindElement("default");
   if (test_element) {
     current_test = new Test;
@@ -151,7 +149,7 @@ bool FGSwitch::Run(void )
 
   if (delay != 0) Delay();
   Clip();
-  if (IsOutput) SetOutput();
+  SetOutput();
 
   return true;
 }
