@@ -8,8 +8,7 @@ namespace yasim {
 
 class Ground {
 public:
-    Ground();
-    virtual ~Ground();
+    virtual ~Ground() = default;
 
     virtual void getGroundPlane(const double pos[3],
                                 double plane[4], float vel[3],
@@ -20,8 +19,8 @@ public:
                                 const simgear::BVHMaterial **material,
                                 unsigned int &body);
 
-    virtual bool getBody(double t, double bodyToWorld[16], double linearVel[3],
-                         double angularVel[3], unsigned int &id);
+   virtual bool getBody(double t, double bodyToWorld[16], double linearVel[3],
+                        double angularVel[3], unsigned int &id);
 
     virtual bool caughtWire(const double pos[4][3]);
 
