@@ -102,7 +102,7 @@ static std::initializer_list<ProgressLabel> progressStrings = {
     {NavDataCache::REBUILD_POIS, QT_TRANSLATE_NOOP("initNavCache","Loading point-of-interest data")}
 };
 
-void initNavCache()
+bool initNavCache()
 {
     const char* baseLabelKey = QT_TRANSLATE_NOOP("initNavCache", "Initialising navigation data, this may take several minutes");
     QString baseLabel= qApp->translate("initNavCache", baseLabelKey);
@@ -126,10 +126,12 @@ void initNavCache()
 
         QTimer updateTimer;
         updateTimer.setInterval(500);
+        bool rebuildIsDone = false;
 
-        QObject::connect(&updateTimer, &QTimer::timeout, [&waitForRebuild]() {
+        QObject::connect(&updateTimer, &QTimer::timeout, [&waitForRebuild, &rebuildIsDone]() {
             if (!NavDataCache::isAnotherProcessRebuilding()) {
                 waitForRebuild.done(0);
+                rebuildIsDone = true;
                 return;
             }
         });
@@ -137,6 +139,12 @@ void initNavCache()
         updateTimer.start(); // timer won't actually run until we process events
         waitForRebuild.exec();
         updateTimer.stop();
+
+        if (!rebuildIsDone) {
+            flightgear::addSentryBreadcrumb("Launcher wait on other process nav-cache rebuild abandoned by user", "info");
+            return false;
+        }
+
         addSentryBreadcrumb("Launcher: done waiting for other process NavCache rebuild dialog", "info");
     }
 
@@ -155,10 +163,13 @@ void initNavCache()
         QTimer updateTimer;
         updateTimer.setInterval(100);
 
-        QObject::connect(&updateTimer, &QTimer::timeout, [&cache, &rebuildProgress, &baseLabel]() {
+        bool didComplete = false;
+
+        QObject::connect(&updateTimer, &QTimer::timeout, [&cache, &rebuildProgress, &baseLabel, &didComplete]() {
             auto phase = cache->rebuild();
             if (phase == NavDataCache::REBUILD_DONE) {
                 rebuildProgress.done(0);
+                didComplete = true;
                 return;
             }
             
@@ -179,12 +190,20 @@ void initNavCache()
                 rebuildProgress.setMaximum(100);
             }
         });
-        
+
         updateTimer.start(); // timer won't actually run until we process events
         rebuildProgress.exec();
         updateTimer.stop();
+
+        if (!didComplete) {
+            flightgear::addSentryBreadcrumb("Launcher nav-cache rebuild abandoned by user", "info");
+            return false;
+        }
+
         flightgear::addSentryBreadcrumb("Launcher nav-cache rebuild complete", "info");
     }
+
+    return true;
 }
 
 class NaturalEarthDataLoaderThread : public QThread
@@ -217,17 +236,9 @@ private:
         if (m_abandoned)
             return;
 
-        flightgear::PolyLineList::const_iterator begin = m_parsedLines.begin() + m_lineInsertCount;
-        unsigned int numToAdd = std::min<unsigned int>(1000U, static_cast<unsigned int>(m_parsedLines.size()) - m_lineInsertCount);
-        flightgear::PolyLineList::const_iterator end = begin + numToAdd;
-        flightgear::PolyLine::bulkAddToSpatialIndex(begin, end);
 
-        if (end == m_parsedLines.end()) {
-            deleteLater(); // commit suicide
-        } else {
-            m_lineInsertCount += 1000;
-            QTimer::singleShot(50, this, SLOT(onFinished()));
-        }
+        flightgear::PolyLine::bulkAddToSpatialIndex(m_parsedLines.begin(), m_parsedLines.end());
+        deleteLater(); // commit suicide
     }
 
     void loadNaturalEarthFile(const std::string& aFileName,
@@ -240,7 +251,6 @@ private:
         if (!path.exists())
             return; // silently fail for now
 
-        flightgear::PolyLineList lines;
         flightgear::SHPParser::parsePolyLines(path, aType, m_parsedLines, areClosed);
     }
 
@@ -249,8 +259,18 @@ private:
     bool m_abandoned = false;
 };
 
-bool checkForWorkingOpenGL()
+enum class OpenGLStatus
 {
+    OpenGL21,
+    Unknown,
+    GDIGeneric,
+    Intel14
+};
+
+OpenGLStatus checkForWorkingOpenGL()
+{
+    // request an OpenGL comptability profile, version 2.1
+    // anything lower and we'll crash
     QSurfaceFormat fmt;
     fmt.setProfile(QSurfaceFormat::CompatibilityProfile);
     fmt.setMajorVersion(2);
@@ -259,23 +279,41 @@ bool checkForWorkingOpenGL()
     QOpenGLContext ctx;
     ctx.setFormat(fmt);
     if (!ctx.create()) {
-        return false;
+        return OpenGLStatus::Unknown;
     }
 
+    // from here on, we need to ensure orderly cleanup or some drivers
+    // crash. So we can't early return.
+
+    OpenGLStatus result = OpenGLStatus::Unknown;
     QOffscreenSurface offSurface;
     offSurface.setFormat(ctx.format()); // ensure it's compatible
     offSurface.create();
-    if (!ctx.makeCurrent(&offSurface)) {
-        return false;
+
+    if (ctx.makeCurrent(&offSurface)) {
+        result = OpenGLStatus::OpenGL21;
+        std::string renderer = (char*)glGetString(GL_RENDERER);
+        if (renderer == "GDI Generic") {
+            flightgear::addSentryBreadcrumb("Detected GDI generic renderer", "info");
+            result = OpenGLStatus::GDIGeneric;
+        } else if (simgear::strutils::starts_with(renderer, "Intel")) {
+            if (ctx.format().majorVersion() < 2) {
+                flightgear::addSentryBreadcrumb("Detected Intel < 2.1 renderer", "info");
+                result = OpenGLStatus::Intel14;
+            }
+        } else if (simgear::strutils::starts_with(renderer, "S3 Graphics")) {
+            if (ctx.format().majorVersion() < 2) {
+                flightgear::addSentryBreadcrumb("Detected S2 < 2.1 renderer", "info");
+                result = OpenGLStatus::Unknown;
+            }
+        }
+
+        // ensure the context is no longer current on the offscreen
+        ctx.doneCurrent();
     }
 
-    std::string renderer = (char*)glGetString(GL_RENDERER);
-    if (renderer == "GDI Generic") {
-        flightgear::addSentryBreadcrumb("Detected GDI generic renderer", "info");
-        return false;
-    }
-
-    return true;
+    offSurface.destroy();
+    return result;
 }
 
 } // of anonymous namespace
@@ -307,7 +345,10 @@ static void simgearMessageOutput(QtMsgType type, const QMessageLogContext &conte
     // important we copy the file name here, since QMessageLogContext doesn't
     sglog().logCopyingFilename(SG_GUI, mappedPriority, file, context.line, s);
     if (type == QtFatalMsg) {
-        abort();
+        // if we abort() here, we get a thread crash which prevents
+        // us actually seeing the error
+        // we'll attempt to continue so we do the SG_POPUP dialog, maybe it works
+        // if it crashes we're no worse off than calling abort() here
     }
 }
 
@@ -322,6 +363,32 @@ namespace flightgear
 
 static std::unique_ptr<QApplication> static_qApp;
 
+
+// becuase QTranslator::load (find_translation, internally) doesn't handle the
+// sciprt part of a language code like: zh-Hans-CN, use this code borrowed from
+// Qt Creator to do the search manually.
+void selectUITranslation()
+{
+    QStringList uiLanguages = QLocale::system().uiLanguages();
+    //qWarning() << "UI languages:" << uiLanguages;
+
+    for (QString locale : qAsConst(uiLanguages)) {
+        // remove script if it exists, eg zh-Hans-CN -> zh-CN
+        locale = QLocale(locale).name();
+        locale.replace('-', '_');
+
+        QTranslator* translator = new QTranslator;
+        ;
+
+        if (translator->load("FlightGear_" + locale, QLatin1String(":/"))) {
+            static_qApp->installTranslator(translator);
+            return;
+        }
+
+        delete translator;
+    }
+}
+
 // Only requires FGGlobals to be initialized if 'doInitQSettings' is true.
 // Safe to call several times.
 void initApp(int& argc, char** argv, bool doInitQSettings)
@@ -357,9 +424,11 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
 		// leave things unset here, so users can use env var
 		// QT_AUTO_SCREEN_SCALE_FACTOR=1 to enable it at runtime
 
-//#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
-      //  QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
-//#endif
+#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+#if !defined (SG_WINDOWS)
+    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
+#endif
         static_qApp.reset(new QApplication(s_argc, argv));
         static_qApp->setOrganizationName("FlightGear");
         static_qApp->setApplicationName("FlightGear");
@@ -377,9 +446,6 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
             static_qApp->installTranslator(fallbackTranslator);
         }
 
-        qInfo() << "UI languages:" << QLocale().uiLanguages();
-        
-        QTranslator* translator = new QTranslator(static_qApp.get());
         // check for --langauge=xx option and prefer that over QLocale
         // detection of the locale if it exists
         auto lang = simgear::strutils::replace(
@@ -387,6 +453,7 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
             "-",
             "_");
         if (!lang.empty()) {
+            QTranslator* translator = new QTranslator(static_qApp.get());
             QString localeFile = "FlightGear_" + QString::fromStdString(lang);
             if (translator->load(localeFile, QLatin1String(":/"))) {
                 qInfo() << "Loaded translations based on --language from:" << localeFile;
@@ -395,13 +462,8 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
                 qInfo() << "--langauge was set, but no translations found at:" << localeFile;
                 delete translator;
             }
-        } else if (translator->load(QLocale(), QLatin1String("FlightGear"), QLatin1String("_"), QLatin1String(":/"))) {
-            // QLocale().name() looks like ' "it_IT" ' (without the outer
-            // quotes) when running FG on Linux with LANG=it_IT.UTF-8.
-            qInfo() << "Loaded translations for locale" << QLocale().name();
-            static_qApp->installTranslator(translator);
         } else {
-            delete translator;
+            selectUITranslation();
         }
 
         // temporary code: only enable getting-started tips for English, to give translators time to
@@ -512,10 +574,41 @@ void restartTheApp()
     qApp->exit(-1);
 }
 
+void startLaunchOnExit(const std::vector<std::string>& originalCommandLine)
+{
+    QStringList fgArgs;
+    for (const auto& arg : originalCommandLine) {
+        fgArgs.append(QString::fromStdString(arg));
+    }
+
+    QProcess proc;
+#if defined(Q_OS_MAC)
+    QDir dir(qApp->applicationDirPath()); // returns the 'MacOS' dir
+    dir.cdUp();                           // up to 'contents' dir
+    dir.cdUp();                           // up to .app dir
+
+    QStringList args;
+    // see 'man open' for details, but '-n' ensures we launch a new instance,
+    // and we want to pass remaining arguments to us, not open.
+    args << "-n" << dir.absolutePath() << "--args" << fgArgs;
+    qDebug() << "args" << args;
+    proc.startDetached("open", args);
+#else
+    proc.startDetached(qApp->applicationFilePath(), fgArgs);
+#endif
+}
+
 void launcherSetSceneryPaths()
 {
     globals->clear_fg_scenery();
 
+    // process path sthe user supplied on the existing command line
+    const auto commandLineSceneryPaths = flightgear::Options::sharedInstance()->valuesForOption("fg-scenery");
+    for (const auto& arg : commandLineSceneryPaths) {
+        // each arg can be multiple paths
+        globals->append_fg_scenery(SGPath::pathsFromUtf8(arg));
+    }
+
 // mimic what options.cxx does, so we can find airport data for parking
 // positions
     QSettings settings;
@@ -546,7 +639,8 @@ void launcherSetSceneryPaths()
 
 bool runLauncherDialog()
 {
-    if (!checkForWorkingOpenGL()) {
+    auto glCheckResult = checkForWorkingOpenGL();
+    if (glCheckResult != OpenGLStatus::OpenGL21) {
         QMessageBox::critical(nullptr, "Failed to find graphics drivers",
                               "This computer is missing suitable graphics drivers (OpenGL) to run FlightGear. "
                               "Please download and install drivers from your graphics card vendor.");
@@ -558,7 +652,10 @@ bool runLauncherDialog()
     // startup the nav-cache now. This pre-empts normal startup of
     // the cache, but no harm done. (Providing scenery paths are consistent)
 
-    initNavCache();
+    bool ok = initNavCache();
+    if (!ok) {
+        return false;
+    }
 
     auto options = flightgear::Options::sharedInstance();
     if (options->isOptionSet("download-dir")) {
@@ -609,10 +706,6 @@ bool runLauncherDialog()
     }
 
     int appResult = qApp->exec();
-    if (appResult <= 0) {
-        return false; // quit
-    }
-
     // avoid crashes / NavCache races if the loader is still running after
     // the launcher exits
     if (naturalEarthLoader) {
@@ -627,6 +720,10 @@ bool runLauncherDialog()
     globals->clear_fg_scenery();
     globals->get_locale()->clear();
 
+    if (appResult <= 0) {
+        return false; // quit
+    }
+    
     return true;
 }
 
