@@ -25,6 +25,7 @@
 
 #include <cstdlib>
 #include <cassert>
+#include <algorithm>
 
 #include <hidapi/hidapi.h>
 #include <hidapi/hidparse.h>
@@ -34,8 +35,6 @@
 #include <simgear/misc/strutils.hxx>
 #include <simgear/io/lowlevel.hxx>
 
-//#define HID_INPUT_DEBUG 1
-
 const char* hexTable = "0123456789ABCDEF";
 
 namespace HID
@@ -161,7 +160,7 @@ namespace HID
                 case AD_AlphanumericDisplay:       return "alphanumeric";
                 case AD_CharacterReport:           return "character-report";
                 case AD_DisplayData:               return "display-data";
-                case AD_DisplayBrightness:         return "display-brightness"; 
+                case AD_DisplayBrightness:         return "display-brightness";
                 case AD_7SegmentDirectMap:          return "seven-segment-direct";
                 case AD_14SegmentDirectMap:         return "fourteen-segment-direct";
 
@@ -245,7 +244,7 @@ public:
     bool Open() override;
     void Close() override;
     void Configure(SGPropertyNode_ptr node) override;
-    
+
     void update(double dt) override;
     const char *TranslateEventName(FGEventData &eventData) override;
     void Send( const char * eventName, double value ) override;
@@ -308,17 +307,21 @@ private:
     int maybeSignExtend(Item* item, int inValue);
 
     void defineReport(SGPropertyNode_ptr reportNode);
-    
+
     std::vector<Report*> _reports;
     std::string _hidPath;
     hid_device* _device = nullptr;
     bool _haveNumberedReports = false;
-    
+    bool _debugRaw = false;
+
     /// set if we parsed the device description our XML
     /// instead of from the USB data. Useful on Windows where the data
     /// is inaccessible, or devices with broken descriptors
     bool _haveLocalDescriptor = false;
-    
+
+    /// allow specifying the descriptor as hex bytes in XML
+    std::vector<uint8_t>_rawXMLDescriptor;
+
     // all sets which will be send on the next update() call.
     std::set<Report*> _dirtyReports;
 };
@@ -341,8 +344,10 @@ FGHIDDevice::FGHIDDevice(hid_device_info *devInfo, FGHIDEventInput *)
 {
     _hidPath = devInfo->path;
 
-    std::wstring manufacturerName,
-        productName = std::wstring(devInfo->product_string);
+    std::wstring manufacturerName, productName;
+    productName = devInfo->product_string ? std::wstring(devInfo->product_string)
+                                          : L"unknown HID device";
+
     if (devInfo->manufacturer_string) {
         manufacturerName = std::wstring(devInfo->manufacturer_string);
         SetName(simgear::strutils::convertWStringToUtf8(manufacturerName) + " " +
@@ -358,9 +363,7 @@ FGHIDDevice::FGHIDDevice(hid_device_info *devInfo, FGHIDEventInput *)
         SetSerialNumber(simgear::strutils::convertWStringToUtf8(serial));
     }
 
-#if defined(HID_INPUT_DEBUG)
-    SG_LOG(SG_INPUT, SG_INFO, "HID device:" << GetName() << " at path " << _hidPath);
-#endif
+    SG_LOG(SG_INPUT, SG_DEBUG, "HID device:" << GetName() << " at path " << _hidPath);
 }
 
 FGHIDDevice::~FGHIDDevice()
@@ -374,31 +377,58 @@ void FGHIDDevice::Configure(SGPropertyNode_ptr node)
 {
     // base class first
     FGInputDevice::Configure(node);
-    
+
     if (node->hasChild("hid-descriptor")) {
         _haveLocalDescriptor = true;
-        SG_LOG(SG_INPUT, SG_INFO, GetUniqueName() << " will configure using local HID descriptor");
-        
+        if (debugEvents) {
+            SG_LOG(SG_INPUT, SG_INFO, GetUniqueName() << " will configure using local HID descriptor");
+        }
+
         for (auto report : node->getChild("hid-descriptor")->getChildren("report")) {
             defineReport(report);
         }
     }
+
+    if (node->hasChild("hid-raw-descriptor")) {
+        _rawXMLDescriptor = simgear::strutils::decodeHex(node->getStringValue("hid-raw-descriptor"));
+        if (debugEvents) {
+            SG_LOG(SG_INPUT, SG_INFO, GetUniqueName() << " will configure using XML-defined raw HID descriptor");
+        }
+    }
+
+    if (node->getBoolValue("hid-debug-raw")) {
+        _debugRaw = true;
+    }
 }
 
 bool FGHIDDevice::Open()
 {
     _device = hid_open_path(_hidPath.c_str());
     if (_device == nullptr) {
-        SG_LOG(SG_INPUT, SG_WARN, "HID: Failed to open:" << _hidPath);
+        SG_LOG(SG_INPUT, SG_WARN, GetUniqueName() << ": HID: Failed to open:" << _hidPath);
+        SG_LOG(SG_INPUT, SG_WARN, "\tnote on Linux you may need to adjust permissions of the device using UDev rules.");
         return false;
     }
 
+#if !defined(SG_WINDOWS)
+    if (_rawXMLDescriptor.empty()) {
+        _rawXMLDescriptor.resize(2048);
+        int descriptorSize = hid_get_descriptor(_device, _rawXMLDescriptor.data(), _rawXMLDescriptor.size());
+        if (descriptorSize <= 0) {
+           SG_LOG(SG_INPUT, SG_WARN, "HID: " << GetUniqueName() << " failed to read HID descriptor");
+           return false;
+        }
+
+        _rawXMLDescriptor.resize(descriptorSize);
+    }
+#endif
+
     if (!_haveLocalDescriptor) {
         bool ok = parseUSBHIDDescriptor();
         if (!ok)
             return false;
     }
-    
+
     for (auto& v : handledEvents) {
         auto reportItem = itemWithName(v.first);
         if (!reportItem.second) {
@@ -408,45 +438,48 @@ bool FGHIDDevice::Open()
 
         FGInputEvent_ptr event = v.second;
         if (debugEvents) {
-            SG_LOG(SG_INPUT, SG_INFO, "found item for event:" << v.first);
+            SG_LOG(SG_INPUT, SG_INFO, "\tfound item for event:" << v.first);
         }
-        
+
         reportItem.second->event = event;
     }
 
     return true;
 }
-    
+
 bool FGHIDDevice::parseUSBHIDDescriptor()
 {
-    unsigned char reportDescriptor[1024];
-    int descriptorSize = hid_get_descriptor(_device, reportDescriptor, 1024);
-    if (descriptorSize <= 0) {
-        SG_LOG(SG_INPUT, SG_WARN, "HID: " << GetUniqueName() << " failed to read HID descriptor");
+#if defined(SG_WINDOWS)
+    if (_rawXMLDescriptor.empty()) {
+        SG_LOG(SG_INPUT, SG_ALERT, GetUniqueName() << ": on Windows, there is no way to extract the UDB-HID report descriptor. "
+               << "\nPlease supply the report descriptor in the device XML configuration.");
+        SG_LOG(SG_INPUT, SG_ALERT, "See this page:<> for information on extracting the report descriptor on Windows");
         return false;
     }
-    
-#if defined(HID_INPUT_DEBUG)
-    SG_LOG(SG_INPUT, SG_INFO, "\nHID: descriptor for:" << GetUniqueName());
-    {
-        std::ostringstream byteString;
-        
-        for (int i=0; i<descriptorSize; ++i) {
-            byteString << hexTable[reportDescriptor[i] >> 4];
-            byteString << hexTable[reportDescriptor[i] & 0x0f];
-            byteString << " ";
+#endif
+
+    if (_debugRaw) {
+        SG_LOG(SG_INPUT, SG_INFO, "\nHID: descriptor for:" << GetUniqueName());
+        {
+            std::ostringstream byteString;
+
+            for (auto i=0; i<_rawXMLDescriptor.size(); ++i) {
+                byteString << hexTable[_rawXMLDescriptor[i] >> 4];
+                byteString << hexTable[_rawXMLDescriptor[i] & 0x0f];
+                byteString << " ";
+            }
+            SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << byteString.str());
         }
-        SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << byteString.str());
     }
-#endif
-    
+
     hid_item* rootItem = nullptr;
-    hid_parse_reportdesc(reportDescriptor, descriptorSize, &rootItem);
-#if defined(HID_INPUT_DEBUG)
-    SG_LOG(SG_INPUT, SG_INFO, "\nHID: scan for:" << GetUniqueName());
-#endif
+    hid_parse_reportdesc(_rawXMLDescriptor.data(), _rawXMLDescriptor.size(), &rootItem);
+    if (debugEvents) {
+        SG_LOG(SG_INPUT, SG_INFO, "\nHID: scan for:" << GetUniqueName());
+    }
+
     parseCollection(rootItem);
-    
+
     hid_free_reportdesc(rootItem);
     return true;
 }
@@ -596,7 +629,7 @@ void FGHIDDevice::update(double dt)
         const uint8_t reportNumber = _haveNumberedReports ? reportBuf[0] : 0;
         auto inputReport = getReport(HID::ReportType::In, reportNumber, false);
         if (!inputReport) {
-            SG_LOG(SG_INPUT, SG_WARN, "FGHIDDevice: Unknown input report number:" << 
+            SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: Unknown input report number:" <<
                 static_cast<int>(reportNumber));
         } else {
             uint8_t* reportBytes = _haveNumberedReports ? reportBuf + 1 : reportBuf;
@@ -636,7 +669,19 @@ void FGHIDDevice::sendReport(Report* report) const
     }
 
     reportLength /= 8;
-// send the data, based on th report type
+
+    if (_debugRaw) {
+        std::ostringstream byteString;
+        for (size_t i=0; i<reportLength; ++i) {
+            byteString << hexTable[reportBytes[i] >> 4];
+            byteString << hexTable[reportBytes[i] & 0x0f];
+            byteString << " ";
+        }
+        SG_LOG(SG_INPUT, SG_INFO, "sending bytes: " << byteString.str());
+    }
+
+
+// send the data, based on the report type
     if (report->type == HID::ReportType::Feature) {
         hid_send_feature_report(_device, reportBytes, reportLength + 1);
     } else {
@@ -654,12 +699,11 @@ void FGHIDDevice::processInputReport(Report* report, unsigned char* data,
                                      size_t length,
                                      double dt, int keyModifiers)
 {
-    if (debugEvents) {
-        SG_LOG(SG_INPUT, SG_INFO, "Report:" << (int) report->number << ", len=" << length);
+    if (_debugRaw) {
+        SG_LOG(SG_INPUT, SG_INFO, GetName() << " FGHIDDeivce received input report:" << (int) report->number << ", len=" << length);
         {
             std::ostringstream byteString;
-        
-            for (int i=0; i<length; ++i) {
+            for (size_t i=0; i<length; ++i) {
                 byteString << hexTable[data[i] >> 4];
                 byteString << hexTable[data[i] & 0x0f];
                 byteString << " ";
@@ -690,7 +734,7 @@ void FGHIDDevice::processInputReport(Report* report, unsigned char* data,
         if (!item->event)
             continue;
 
-        if (debugEvents) {
+        if (_debugRaw) {
             SG_LOG(SG_INPUT, SG_INFO, "\titem:" << item->name << " = " << value);
         }
 
@@ -705,11 +749,29 @@ void FGHIDDevice::SendFeatureReport(unsigned int reportId, const std::string& da
         return;
     }
 
+    if (_debugRaw) {
+        SG_LOG(SG_INPUT, SG_INFO, GetName() << ": FGHIDDevice: Sending feature report:" << (int) reportId << ", len=" << data.size());
+        {
+            std::ostringstream byteString;
+
+            for (unsigned int i=0; i<data.size(); ++i) {
+                byteString << hexTable[data[i] >> 4];
+                byteString << hexTable[data[i] & 0x0f];
+                byteString << " ";
+            }
+            SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << byteString.str());
+        }
+    }
+
     uint8_t buf[65];
     size_t len = std::min(data.length() + 1, sizeof(buf));
     buf[0] = reportId;
     memcpy(buf + 1, data.data(), len - 1);
-    hid_send_feature_report(_device, buf, len);
+    size_t r = hid_send_feature_report(_device, buf, len);
+    if (r < 0) {
+        SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: Sending feature report failed, error-string is:\n"
+               << simgear::strutils::error_string(errno));
+    }
 }
 
 const char *FGHIDDevice::TranslateEventName(FGEventData &eventData)
@@ -722,7 +784,7 @@ void FGHIDDevice::Send(const char *eventName, double value)
 {
     auto item = itemWithName(eventName);
     if (item.second == nullptr) {
-        SG_LOG(SG_INPUT, SG_WARN, "FGHIDDevice: unknown item name:" << eventName);
+        SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice:unknown item name:" << eventName);
         return;
     }
 
@@ -735,14 +797,14 @@ void FGHIDDevice::Send(const char *eventName, double value)
     item.second->lastValue = intValue;
     _dirtyReports.insert(item.first);
 }
-    
+
 void FGHIDDevice::defineReport(SGPropertyNode_ptr reportNode)
 {
     const int nChildren = reportNode->nChildren();
     uint32_t bitCount = 0;
     const auto rty = HID::reportTypeFromString(reportNode->getStringValue("type"));
     if (rty == HID::ReportType::Invalid) {
-        SG_LOG(SG_INPUT, SG_WARN, "FGHIDDevice: invalid report type:" <<
+        SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: invalid report type:" <<
                reportNode->getStringValue("type"));
         return;
     }
@@ -762,21 +824,21 @@ void FGHIDDevice::defineReport(SGPropertyNode_ptr reportNode)
             bitCount += size;
             continue;
         }
-        
+
         if (!strcmp(nd->getName(), "type") || !strcmp(nd->getName(), "id")) {
             continue; // already handled above
         }
-        
+
         // allow repeating items
         uint8_t count = nd->getIntValue("count", 1);
         std::string name = nd->getNameString();
         const auto lastHypen = name.rfind("-");
         std::string baseName = name.substr(0, lastHypen + 1);
         int baseIndex = std::stoi(name.substr(lastHypen + 1));
-        
+
         const bool isRelative = (name.find("rel-") == 0);
         const bool isSigned = nd->getBoolValue("is-signed", false);
-        
+
         for (uint8_t i=0; i < count; ++i) {
             std::ostringstream oss;
             oss << baseName << (baseIndex + i);
@@ -893,13 +955,14 @@ void FGHIDEventInput::update(double dt)
     FGEventInput::update(dt);
 }
 
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGHIDEventInput> registrantFGHIDEventInput;
+
 ///////////////////////////////////////////////////////////////////////////////////////////////
 
 void FGHIDEventInput::FGHIDEventInputPrivate::evaluateDevice(hid_device_info* deviceInfo)
 {
-#if defined(HID_INPUT_DEBUG)
-    SG_LOG(SG_INPUT, SG_INFO, "HID device:" << deviceInfo->product_string << " from " << deviceInfo->manufacturer_string);
-#endif
     // allocate an input device, and add to the base class to see if we have
     // a config
     p->AddDevice(new FGHIDDevice(deviceInfo, p));
