@@ -154,6 +154,14 @@ static string cleanRunwayNo(const string& aRwyNo)
   return result;
 }
 
+class AbandonCacheException : public sg_exception
+{
+public:
+    AbandonCacheException() : sg_exception("Cache build cancelled", {}, {}, false)
+    {
+    }
+};
+
 } // anonymous namespace
 
 namespace flightgear
@@ -178,6 +186,11 @@ public:
 
   }
 
+  ~RebuildThread()
+  {
+      join();
+  }
+
   bool isFinished() const
   {
     std::lock_guard<std::mutex> g(_lock);
@@ -188,12 +201,14 @@ public:
   {
     SGTimeStamp st;
     st.stamp();
+    flightgear::addSentryBreadcrumb("Rebuild thread running", "info");
     _cache->doRebuild();
     SG_LOG(SG_NAVCACHE, SG_INFO, "cache rebuild took:" << st.elapsedMSec() << "msec");
 
     std::lock_guard<std::mutex> g(_lock);
     _isFinished = true;
     _phase = NavDataCache::REBUILD_DONE;
+    flightgear::addSentryBreadcrumb("Rebuild thread done", "info");
   }
 
     NavDataCache::RebuildPhase currentPhase() const
@@ -290,10 +305,6 @@ public:
           throw sg_exception("Nav-cache file is not writeable");
       }
 
-      if (outer->isAnotherProcessRebuilding()) {
-          SG_LOG(SG_NAVCACHE, SG_ALERT, "NavDataCache: init() while another processing is rebuilding the DB");
-      }
-
       int openFlags = readOnly ? SQLITE_OPEN_READONLY :
         (SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE);
       std::string pathUtf8 = path.utf8Str();
@@ -341,7 +352,7 @@ public:
       int schemaVersion = outer->readIntProperty("schema-version");
       if (schemaVersion != SCHEMA_VERSION) {
         SG_LOG(SG_NAVCACHE, SG_INFO, "Navcache schema mismatch, will rebuild");
-        throw sg_exception("Navcache schema has changed");
+          throw sg_exception("Navcache schema has changed", {}, {}, false);
       }
     }
 
@@ -388,28 +399,45 @@ public:
     NavDataCache::DatFileType datFileType,
     bool verbose);
 
-  void callSqlite(int result, const string& sql)
-  {
-    if (result == SQLITE_OK)
-      return; // all good
+    
+    sqlite3_stmt_ptr prepareSQL(const std::string& sql)
+    {
+        sqlite3_stmt_ptr stmt;
+        int result = sqlite3_prepare_v2(db, sql.c_str(), sql.length(), &stmt, nullptr);
+        int retries = 0;
+        int retryMSec = 1;
+        
+        while (result == SQLITE_BUSY) {
+            if (retries > MAX_RETRIES) {
+                break;
+            }
+            
+            ++retries;
+            SGTimeStamp::sleepForMSec(retryMSec);
+            retryMSec = retryMSec << 1; // double each time
+            // try again
+            result = sqlite3_prepare_v2(db, sql.c_str(), sql.length(), &stmt, nullptr);
+        }
+        
+        if (result == SQLITE_OK) {
+            return stmt; // common case, all good
+        }
+        
+      string errMsg;
+      if (result == SQLITE_MISUSE) {
+        errMsg = "Sqlite API abuse";
+        SG_LOG(SG_NAVCACHE, SG_ALERT, "Sqlite API abuse");
+      } else {
+        errMsg = sqlite3_errmsg(db);
+        SG_LOG(SG_NAVCACHE, SG_ALERT, "Sqlite error:" << errMsg << " running:\n\t" << sql);
+      }
 
-    string errMsg;
-    if (result == SQLITE_MISUSE) {
-      errMsg = "Sqlite API abuse";
-      SG_LOG(SG_NAVCACHE, SG_ALERT, "Sqlite API abuse");
-    } else {
-      errMsg = sqlite3_errmsg(db);
-      SG_LOG(SG_NAVCACHE, SG_ALERT, "Sqlite error:" << errMsg << " running:\n\t" << sql);
+      throw sg_exception("Sqlite error:" + errMsg, sql);
     }
 
-    throw sg_exception("Sqlite error:" + errMsg, sql);
-  }
-
   void runSQL(const string& sql)
   {
-    sqlite3_stmt_ptr stmt;
-    callSqlite(sqlite3_prepare_v2(db, sql.c_str(), sql.length(), &stmt, NULL), sql);
-
+      sqlite3_stmt_ptr stmt = prepareSQL(sql);
     try {
       execSelect(stmt);
     } catch (sg_exception&) {
@@ -422,8 +450,7 @@ public:
 
   sqlite3_stmt_ptr prepare(const string& sql)
   {
-    sqlite3_stmt_ptr stmt;
-    callSqlite(sqlite3_prepare_v2(db, sql.c_str(), sql.length(), &stmt, NULL), sql);
+      sqlite3_stmt_ptr stmt = prepareSQL(sql);
     prepared.push_back(stmt);
     return stmt;
   }
@@ -456,25 +483,31 @@ public:
 
   bool stepSelect(sqlite3_stmt_ptr stmt)
   {
-    int retries = 0;
-    int result;
-    while (retries < MAX_RETRIES) {
-      result = sqlite3_step(stmt);
-      if (result == SQLITE_ROW) {
-        return true; // at least one result row
-      }
+      if (abandonCache)
+          throw AbandonCacheException{};
 
-      if (result == SQLITE_DONE) {
-        return false; // no result rows
-      }
+      int retries = 0;
+      int result;
+      int retryMSec = 10;
 
-      if (result != SQLITE_BUSY) {
-        break;
-      }
+      while (retries < MAX_RETRIES) {
+          result = sqlite3_step(stmt);
+          if (result == SQLITE_ROW) {
+              return true; // at least one result row
+          }
 
-      SG_LOG(SG_NAVCACHE, SG_ALERT, "NavCache contention on select, will retry:" << retries);
-      SGTimeStamp::sleepForMSec(++retries * 10);
-    } // of retry loop for DB locked
+          if (result == SQLITE_DONE) {
+              return false; // no result rows
+          }
+
+          if (result != SQLITE_BUSY) {
+              break;
+          }
+
+          SG_LOG(SG_NAVCACHE, SG_ALERT, "NavCache contention on select, will retry:" << retries);
+          SGTimeStamp::sleepForMSec(retryMSec);
+          retryMSec *= 2; // double the back-off time, each time
+      }                   // of retry loop for DB locked
 
     if (retries >= MAX_RETRIES) {
       SG_LOG(SG_NAVCACHE, SG_ALERT, "exceeded maximum number of SQLITE_BUSY retries");
@@ -786,23 +819,26 @@ public:
                                 const string& name, const SGGeod& pos, PositionedID apt,
                                 bool spatialIndex)
   {
-    SGVec3d cartPos(SGVec3d::fromGeod(pos));
+      if (abandonCache)
+          throw AbandonCacheException{};
 
-    sqlite3_bind_int(insertPositionedQuery, 1, ty);
-    sqlite_bind_stdstring(insertPositionedQuery, 2, ident);
-    sqlite_bind_stdstring(insertPositionedQuery, 3, name);
-    sqlite3_bind_int64(insertPositionedQuery, 4, apt);
-    sqlite3_bind_double(insertPositionedQuery, 5, pos.getLongitudeDeg());
-    sqlite3_bind_double(insertPositionedQuery, 6, pos.getLatitudeDeg());
-    sqlite3_bind_double(insertPositionedQuery, 7, pos.getElevationM());
+      SGVec3d cartPos(SGVec3d::fromGeod(pos));
 
-    if (spatialIndex) {
-      Octree::Leaf* octreeLeaf = Octree::global_spatialOctree->findLeafForPos(cartPos);
-      assert(intersects(octreeLeaf->bbox(), cartPos));
-      sqlite3_bind_int64(insertPositionedQuery, 8, octreeLeaf->guid());
-    } else {
-      sqlite3_bind_null(insertPositionedQuery, 8);
-    }
+      sqlite3_bind_int(insertPositionedQuery, 1, ty);
+      sqlite_bind_stdstring(insertPositionedQuery, 2, ident);
+      sqlite_bind_stdstring(insertPositionedQuery, 3, name);
+      sqlite3_bind_int64(insertPositionedQuery, 4, apt);
+      sqlite3_bind_double(insertPositionedQuery, 5, pos.getLongitudeDeg());
+      sqlite3_bind_double(insertPositionedQuery, 6, pos.getLatitudeDeg());
+      sqlite3_bind_double(insertPositionedQuery, 7, pos.getElevationM());
+
+      if (spatialIndex) {
+          Octree::Leaf* octreeLeaf = Octree::globalPersistentOctree()->findLeafForPos(cartPos);
+          assert(intersects(octreeLeaf->bbox(), cartPos));
+          sqlite3_bind_int64(insertPositionedQuery, 8, octreeLeaf->guid());
+      } else {
+          sqlite3_bind_null(insertPositionedQuery, 8);
+      }
 
     sqlite3_bind_double(insertPositionedQuery, 9, cartPos.x());
     sqlite3_bind_double(insertPositionedQuery, 10, cartPos.y());
@@ -897,68 +933,73 @@ public:
   SGPath path;
     bool readOnly;
 
-  /// the actual cache of ID -> instances. This holds an owning reference,
-  /// so once items are in the cache they will never be deleted until
-  /// the cache drops its reference
-  PositionedCache cache;
-  unsigned int cacheHits, cacheMisses;
+    // flag set during shutdown: allows us to abandon queries, etc
+    // if exit is requested during a rebuild.
+    bool abandonCache = false;
 
-  /**
+
+    /// the actual cache of ID -> instances. This holds an owning reference,
+    /// so once items are in the cache they will never be deleted until
+    /// the cache drops its reference
+    PositionedCache cache;
+    unsigned int cacheHits, cacheMisses;
+
+    /**
    * record the levels of open transaction objects we have
    */
-  unsigned int transactionLevel;
-  bool transactionAborted;
-  sqlite3_stmt_ptr beginTransactionStmt, commitTransactionStmt, rollbackTransactionStmt;
+    unsigned int transactionLevel;
+    bool transactionAborted;
+    sqlite3_stmt_ptr beginTransactionStmt, commitTransactionStmt, rollbackTransactionStmt;
 
-  std::map<DatFileType, NavDataCache::DatFilesGroupInfo> datFilesInfo;
-  SGPath metarDatPath, poiDatPath,
-  carrierDatPath, airwayDatPath;
+    std::map<DatFileType, NavDataCache::DatFilesGroupInfo> datFilesInfo;
+    SGPath metarDatPath, poiDatPath,
+        carrierDatPath, airwayDatPath;
 
-  sqlite3_stmt_ptr readPropertyQuery, writePropertyQuery,
-    stampFileCache, statCacheCheck,
-    loadAirportStmt, loadCommStation, loadPositioned, loadNavaid,
-    loadRunwayStmt;
-  sqlite3_stmt_ptr writePropertyMulti, clearProperty;
+    sqlite3_stmt_ptr readPropertyQuery, writePropertyQuery,
+        stampFileCache, statCacheCheck,
+        loadAirportStmt, loadCommStation, loadPositioned, loadNavaid,
+        loadRunwayStmt;
+    sqlite3_stmt_ptr writePropertyMulti, clearProperty;
 
-  sqlite3_stmt_ptr insertPositionedQuery, insertAirport, insertTower, insertRunway,
-  insertCommStation, insertNavaid;
-  sqlite3_stmt_ptr setAirportMetar, setRunwayReciprocal, setRunwayILS, setNavaidColocated,
-    setAirportPos;
-  sqlite3_stmt_ptr removePOIQuery;
+    sqlite3_stmt_ptr insertPositionedQuery, insertAirport, insertTower, insertRunway,
+        insertCommStation, insertNavaid;
+    sqlite3_stmt_ptr setAirportMetar, setRunwayReciprocal, setRunwayILS, setNavaidColocated,
+        setAirportPos;
+    sqlite3_stmt_ptr removePOIQuery;
 
-  sqlite3_stmt_ptr findClosestWithIdent;
-// octree (spatial index) related queries
-  sqlite3_stmt_ptr getOctreeChildren, insertOctree, updateOctreeChildren,
-    getOctreeLeafChildren;
+    sqlite3_stmt_ptr findClosestWithIdent;
+    // octree (spatial index) related queries
+    sqlite3_stmt_ptr getOctreeChildren, insertOctree, updateOctreeChildren,
+        getOctreeLeafChildren;
 
-  sqlite3_stmt_ptr searchAirports, getAllAirports;
-  sqlite3_stmt_ptr findCommByFreq, findNavsByFreq,
-  findNavsByFreqNoPos, findNavaidForRunway;
-  sqlite3_stmt_ptr getAirportItems, getAirportItemByIdent;
-  sqlite3_stmt_ptr findAirportRunway,
-    findILS;
+    sqlite3_stmt_ptr searchAirports, getAllAirports;
+    sqlite3_stmt_ptr findCommByFreq, findNavsByFreq,
+        findNavsByFreqNoPos, findNavaidForRunway;
+    sqlite3_stmt_ptr getAirportItems, getAirportItemByIdent;
+    sqlite3_stmt_ptr findAirportRunway,
+        findILS;
 
-  sqlite3_stmt_ptr runwayLengthFtQuery;
+    sqlite3_stmt_ptr runwayLengthFtQuery;
 
-// airways
-  sqlite3_stmt_ptr findAirway, findAirwayNet, insertAirwayEdge,
-    isPosInAirway, airwayEdgesFrom, airwayEdgesTo,
-    insertAirway, airwayEdges;
-  sqlite3_stmt_ptr loadAirway;
+    // airways
+    sqlite3_stmt_ptr findAirway, findAirwayNet, insertAirwayEdge,
+        isPosInAirway, airwayEdgesFrom, airwayEdgesTo,
+        insertAirway, airwayEdges;
+    sqlite3_stmt_ptr loadAirway;
 
-// since there's many permutations of ident/name queries, we create
-// them programtically, but cache the exact query by its raw SQL once
-// used.
-  std::map<string, sqlite3_stmt_ptr> findByStringDict;
+    // since there's many permutations of ident/name queries, we create
+    // them programtically, but cache the exact query by its raw SQL once
+    // used.
+    std::map<string, sqlite3_stmt_ptr> findByStringDict;
 
-  typedef std::vector<sqlite3_stmt_ptr> StmtVec;
-  StmtVec prepared;
+    typedef std::vector<sqlite3_stmt_ptr> StmtVec;
+    StmtVec prepared;
 
-  std::set<Octree::Branch*> deferredOctreeUpdates;
+    std::set<Octree::Branch*> deferredOctreeUpdates;
 
-  // if we're performing a rebuild, the thread that is doing the work.
-  // otherwise, NULL
-  std::unique_ptr<RebuildThread> rebuilder;
+    // if we're performing a rebuild, the thread that is doing the work.
+    // otherwise, NULL
+    std::unique_ptr<RebuildThread> rebuilder;
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -966,25 +1007,27 @@ public:
 FGPositioned* NavDataCache::NavDataCachePrivate::loadById(sqlite3_int64 rowid,
                                                           sqlite3_int64& aptId)
 {
+    if (abandonCache)
+        throw AbandonCacheException{};
 
-  sqlite3_bind_int64(loadPositioned, 1, rowid);
-  execSelect1(loadPositioned);
+    sqlite3_bind_int64(loadPositioned, 1, rowid);
+    execSelect1(loadPositioned);
 
-  assert(rowid == sqlite3_column_int64(loadPositioned, 0));
-  FGPositioned::Type ty = (FGPositioned::Type) sqlite3_column_int(loadPositioned, 1);
+    assert(rowid == sqlite3_column_int64(loadPositioned, 0));
+    FGPositioned::Type ty = (FGPositioned::Type)sqlite3_column_int(loadPositioned, 1);
 
-  PositionedID prowid = static_cast<PositionedID>(rowid);
-  string ident = (char*) sqlite3_column_text(loadPositioned, 2);
-  string name = (char*) sqlite3_column_text(loadPositioned, 3);
-  aptId = sqlite3_column_int64(loadPositioned, 4);
-  double lon = sqlite3_column_double(loadPositioned, 5);
-  double lat = sqlite3_column_double(loadPositioned, 6);
-  double elev = sqlite3_column_double(loadPositioned, 7);
-  SGGeod pos = SGGeod::fromDegM(lon, lat, elev);
+    PositionedID prowid = static_cast<PositionedID>(rowid);
+    string ident = (char*)sqlite3_column_text(loadPositioned, 2);
+    string name = (char*)sqlite3_column_text(loadPositioned, 3);
+    aptId = sqlite3_column_int64(loadPositioned, 4);
+    double lon = sqlite3_column_double(loadPositioned, 5);
+    double lat = sqlite3_column_double(loadPositioned, 6);
+    double elev = sqlite3_column_double(loadPositioned, 7);
+    SGGeod pos = SGGeod::fromDegM(lon, lat, elev);
 
-  reset(loadPositioned);
+    reset(loadPositioned);
 
-  switch (ty) {
+    switch (ty) {
     case FGPositioned::AIRPORT:
     case FGPositioned::SEAPORT:
     case FGPositioned::HELIPORT:
@@ -1036,7 +1079,7 @@ FGPositioned* NavDataCache::NavDataCachePrivate::loadById(sqlite3_int64 rowid,
 
     default:
       return NULL;
-  }
+    }
 }
 
 bool NavDataCache::NavDataCachePrivate::isCachedFileModified(const SGPath& path, bool verbose)
@@ -1256,6 +1299,7 @@ NavDataCache::NavDataCache()
 
     // permit additional DB connections from the same process
     sqlite3_config(SQLITE_CONFIG_MULTITHREAD);
+    const bool readOnly = fgGetBool("/sim/fghome-readonly", false);
 
     for (int t=0; t < MAX_TRIES; ++t) {
         SGPath cachePath = homePath / os.str();
@@ -1269,6 +1313,13 @@ NavDataCache::NavDataCache()
             SG_LOG(SG_NAVCACHE, t == 0 ? SG_WARN : SG_ALERT, "NavCache: init failed:" << e.what()
                    << " (attempt " << t << ")");
 
+            if (readOnly) {
+              flightgear::fatalMessageBoxThenExit(
+                  "Unable to open navigation cache",
+                  std::string("The navigation data cache could not be opened, and this copy of FlightGear is runnning read-only. FlightGear needs to be started with write permissions to rebuild the cache.")
+                  + e.getMessage(), e.getOrigin());
+            }
+
             if (t == (MAX_TRIES - 1)) {
                 // final attempt still failed, we are busted
                 flightgear::fatalMessageBoxThenExit(
@@ -1280,7 +1331,7 @@ NavDataCache::NavDataCache()
             d.reset();
 
             // only wipe the existing if not readonly
-            if (cachePath.exists() && !fgGetBool("/sim/fghome-readonly", false)) {
+            if (cachePath.exists() && !readOnly) {
                 bool ok = cachePath.remove();
                 if (!ok) {
                     SG_LOG(SG_NAVCACHE, SG_ALERT, "NavCache: failed to remove previous cache file");
@@ -1293,11 +1344,6 @@ NavDataCache::NavDataCache()
         }
     } // of retry loop
 
-    double RADIUS_EARTH_M = 7000 * 1000.0; // 7000km is plenty
-    SGVec3d earthExtent(RADIUS_EARTH_M, RADIUS_EARTH_M, RADIUS_EARTH_M);
-    Octree::global_spatialOctree =
-    new Octree::Branch(SGBox<double>(-earthExtent, earthExtent), 1);
-
     // Update d->aptDatFilesInfo, d->metarDatPath, d->navDatPath, etc.
     updateListsOfDatFiles();
 }
@@ -1305,12 +1351,24 @@ NavDataCache::NavDataCache()
 NavDataCache::~NavDataCache()
 {
   assert(static_instance == this);
-  static_instance = nullptr;
-  d.reset();
+
+  if (d->rebuilder) {
+      addSentryBreadcrumb("shutting down cache with rebuild active", "info");
+      // setting thsi will cause DB operations to throw the special
+      // AbandonCache exception, and hence cause the rebuild thread to
+      // exit pretty quickly, so the join() won't take too long.
+      d->abandonCache = true;
+      d->rebuilder.reset(); // will the destructor which does a join()
+      addSentryBreadcrumb("abandoned rebuild scuessfully", "info");
+  }
+
 
 // ensure we wip the airports cache too, or we'll get out
 // of sync during tests
   FGAirport::clearAirportsCache();
+
+  static_instance = nullptr;
+  d.reset();
 }
 
 NavDataCache* NavDataCache::createInstance()
@@ -1325,6 +1383,13 @@ NavDataCache* NavDataCache::instance()
   return static_instance;
 }
 
+void NavDataCache::shutdown()
+{
+    if (static_instance) {
+        delete static_instance;
+    }
+}
+
 // Update the lists of dat files used for NavCache freshness checking and
 // rebuilding.
 void NavDataCache::updateListsOfDatFiles() {
@@ -1483,6 +1548,7 @@ NavDataCache::RebuildPhase NavDataCache::rebuild()
                                                 "Failed to initialise NavCache rebuild protection");
         }
 
+        flightgear::addSentryBreadcrumb("NavCache accquired rebuild lock", "info");
         d->rebuilder.reset(new RebuildThread(this));
         d->rebuilder->start();
     }
@@ -1492,6 +1558,7 @@ NavDataCache::RebuildPhase NavDataCache::rebuild()
     if (phase == REBUILD_DONE) {
         d->rebuilder.reset(); // all done!
         releaseRebuildLock();
+        flightgear::addSentryBreadcrumb("NavCache released rebuild lock", "info");
     }
     return phase;
 }
@@ -1521,6 +1588,7 @@ bool NavDataCache::isAnotherProcessRebuilding()
         // (there could be multiple read-only copies in this situation)
         ReleaseMutex(static_fgNavCacheRebuildMutex);
         CloseHandle(static_fgNavCacheRebuildMutex);
+		static_fgNavCacheRebuildMutex = nullptr;
         return false;
     }
 
@@ -1610,6 +1678,7 @@ void NavDataCache::doRebuild()
   rebuildInProgress = true;
 
   try {
+    flightgear::addSentryBreadcrumb("NavDataCache::doRebuild starts", "info");
     d->close(); // completely close the sqlite object
     d->path.remove(); // remove the file on disk
     d->init(); // start again from scratch
@@ -1692,6 +1761,7 @@ void NavDataCache::doRebuild()
 
       }
 
+      flightgear::addSentryBreadcrumb("NavDataCache::doRebuild ends", "info");
   } catch (sg_exception& e) {
     SG_LOG(SG_NAVCACHE, SG_ALERT, "caught exception rebuilding navCache:" << e.what());
   }
@@ -1995,7 +2065,7 @@ void NavDataCache::updatePosition(PositionedID item, const SGGeod &pos)
 // structures alone. This is fine providing items do no move very far, since
 // all the spatial searches ultimately use the items' real cartesian position,
 // which was updated above.
-  Octree::Leaf* octreeLeaf = Octree::global_spatialOctree->findLeafForPos(cartPos);
+  Octree::Leaf* octreeLeaf = Octree::globalPersistentOctree()->findLeafForPos(cartPos);
   sqlite3_bind_int64(d->setAirportPos, 5, octreeLeaf->guid());
 
   sqlite3_bind_double(d->setAirportPos, 6, cartPos.x());
@@ -2718,7 +2788,7 @@ NavDataCache::Transaction::Transaction(NavDataCache* cache) :
 
 NavDataCache::Transaction::~Transaction()
 {
-    if (_instance->isReadOnly()) {
+    if (_instance->d->abandonCache || _instance->isReadOnly()) {
         return;
     }
 
@@ -2794,8 +2864,11 @@ NavDataCache::ThreadedGUISearch::ThreadedGUISearch(const std::string& term, bool
         sql = "SELECT rowid FROM positioned WHERE name LIKE '%" + term
                 + "%' AND (type >= 1 AND type <= 3)";
     } else {
+        // types are hard-coded here becuase this is only used by NavaidSearchModel
+        // in ther launcher. We would ideally use a TypeFilter but that would
+        // mean loading each positioned to filter them, which is inefficient.
         sql = "SELECT rowid FROM positioned WHERE name LIKE '%" + term
-                + "%' AND ((type >= 1 AND type <= 3) OR ((type >= 9 AND type <= 11))) ";
+                + "%' AND ((type >= 1 AND type <= 3) OR ((type >= 9 AND type <= 11)) OR (type=18 AND name LIKE '% TACAN') ) ";
     }
     sqlite3_prepare_v2(d->db, sql.c_str(), sql.length(), &d->query, NULL);
 
