@@ -29,6 +29,8 @@
 
 class FGSwiftAircraftManager
 {
+    using FGAISwiftAircraftPtr = SGSharedPtr<FGAISwiftAircraft>;
+
 public:
     FGSwiftAircraftManager();
     ~FGSwiftAircraftManager();
@@ -38,11 +40,13 @@ public:
                                                         std::vector<double>& elevationsM, std::vector<double>& verticalOffsets) const;
     void                          removePlane(const std::string& callsign);
 	void removeAllPlanes();
+    void setPlanesTransponders(const std::vector<AircraftTransponder>& transponders);
 	double getElevationAtPosition(const std::string &callsign, const SGGeod& pos) const;
     bool                                                isInitialized() const;
+    void setPlanesSurfaces(const std::vector<AircraftSurfaces>& surfaces);
 
 private:
-    std::unordered_map<std::string, FGAISwiftAircraft*> aircraftByCallsign;
+    std::unordered_map<std::string, FGAISwiftAircraftPtr> aircraftByCallsign;
     bool m_initialized = false;
 
 };
