@@ -38,6 +38,7 @@
 #include <ATC/CommStation.hxx>
 #include <Airports/airport.hxx>
 #include <Navaids/navlist.hxx>
+#include <Main/sentryIntegration.hxx>
 
 #include <3rdparty/iaxclient/lib/iaxclient.h>
 
@@ -179,12 +180,24 @@ void FGCom::bind()
   _ptt_node->addChangeListener(this);
   _selected_comm_node->addChangeListener(this);
   _test_node->addChangeListener(this);
+  // if you listen to more properties, be sure to remove
+  // the listener in unbind() as well
 }
 
 
 
 void FGCom::unbind()
 {
+    _selectedOutput_node->removeChangeListener(this);
+    _selectedInput_node->removeChangeListener(this);
+    _speakerLevel_node->removeChangeListener(this);
+    _silenceThd_node->removeChangeListener(this);
+    _micBoost_node->removeChangeListener(this);
+    _micLevel_node->removeChangeListener(this);
+    _enabled_node->removeChangeListener(this);
+    _ptt_node->removeChangeListener(this);
+    _selected_comm_node->removeChangeListener(this);
+    _test_node->removeChangeListener(this);
 }
 
 
@@ -469,6 +482,9 @@ void FGCom::shutdown()
   iaxc_stop_processing_thread();
   iaxc_shutdown();
 
+    // added to help debugging lingering IAX thread after fgOSCloseWindow
+    flightgear::addSentryBreadcrumb("Did shutdown FGCom", "info");
+    
   assert( static_instance == this );
   static_instance = NULL;
 }
@@ -490,6 +506,12 @@ void FGCom::valueChanged(SGPropertyNode *prop)
     return;
   }
 
+  // avoid crash when properties change before FGCom::postinit
+  // https://sourceforge.net/p/flightgear/codetickets/2574/
+  if (!_initialized) {
+      return;
+  }
+
   if (prop == _commVolumeNode && _enabled) {
       if (_ptt_node->getIntValue()) {
           SG_LOG(SG_IO, SG_INFO, "FGCom: ignoring change comm volume as PTT pressed");
