@@ -107,10 +107,10 @@
 #include "renderer.hxx"
 #include "CameraGroup.hxx"
 #include "FGEventHandler.hxx"
+#include <Main/sentryIntegration.hxx>
 
-#if defined(HAVE_QT)
+#if defined(ENABLE_QQ_UI)
 #include <GUI/QQuickDrawable.hxx>
-#include <Viewer/GraphicsWindowQt5.hxx>
 #endif
 
 #if defined(HAVE_PUI)
@@ -519,16 +519,29 @@ FGRenderer::setupView( void )
     Ephemeris* ephemerisSub = globals->get_subsystem<Ephemeris>();
 
 
-    // The sun and moon diameters are scaled down numbers of the
-    // actual diameters. This was needed to fit both the sun and the
-    // moon within the distance to the far clip plane.
-    // Moon diameter:    3,476 kilometers
-    // Sun diameter: 1,390,000 kilometers
+    // The sun and moon radius are scaled down numbers of the actual
+    // diameters. This is needed to fit both the sun and the moon
+    // within the distance to the far clip plane.
+    // 
+    // Their distance to the observer is specified in FGRendered::updateSky() and
+    // set to 40000 for the Moon semi-mayor axis, 50000 for the mean
+    // Earth-Sun distance (1AU). The location of the stars is
+    // specified below in the call to sky->build() (80000).
+    //
+    // Mean Moon radius: 1,737.4 kilometers
+    // Moon Semi-mayor axis: 384,399 km
+    // => Rendered Moon radius = 1,737.4 / 384,399 * 40000 = 232.5
+    //
+    // Photosphere Sun radius: 695,700 kilometers
+    // 1UA = 149,597,870.700 km
+    // => Rendered Sun radius = 695,700/149,597,870.700 * 50000 = 180.8
+    //
+
     osg::ref_ptr<simgear::SGReaderWriterOptions> opt;
     opt = simgear::SGReaderWriterOptions::fromPath(globals->get_fg_root());
     opt->setPropertyNode(globals->get_props());
     _sky->build( 80000.0, 80000.0,
-                  463.3, 361.8,
+                  232.5, 180.8,
                   *ephemerisSub->data(),
                   fgGetNode("/environment", true),
                   opt.get());
@@ -601,31 +614,32 @@ FGRenderer::setupView( void )
     stateSet->setAttributeAndModes(fog);
     stateSet->setUpdateCallback(new FGFogEnableUpdateCallback);
 
-    // plug in the GUI
     osg::Camera* guiCamera = getGUICamera(CameraGroup::getDefault());
     if (guiCamera) {
-        osg::Geode* geode = new osg::Geode;
-        geode->addDrawable(new SGHUDDrawable);
+        osg::Geode* hudGeode = new osg::Geode;
+        hudGeode->addDrawable(new SGHUDDrawable);
+        guiCamera->addChild(hudGeode);
 
 #if defined(HAVE_PUI)
         _puiCamera = new flightgear::PUICamera;
-        _puiCamera->init(guiCamera);
+        _puiCamera->init(guiCamera, viewer);
 #endif
 
-#if defined(HAVE_QT)
+#if defined(ENABLE_QQ_UI)
         std::string rootQMLPath = fgGetString("/sim/gui/qml-root-path");
-        auto graphicsWindowQt = dynamic_cast<GraphicsWindowQt5*>(guiCamera->getGraphicsContext());
+        auto graphicsWindow = dynamic_cast<osgViewer::GraphicsWindow*>(guiCamera->getGraphicsContext());
 
-        if (graphicsWindowQt && !rootQMLPath.empty()) {
+        if (!rootQMLPath.empty()) {
             _quickDrawable = new QQuickDrawable;
-            _quickDrawable->setup(graphicsWindowQt);
+            _quickDrawable->setup(graphicsWindow, viewer);
 
             _quickDrawable->setSource(QUrl::fromLocalFile(QString::fromStdString(rootQMLPath)));
-            geode->addDrawable(_quickDrawable);
+            
+            osg::Geode* qqGeode = new osg::Geode;
+            qqGeode->addDrawable(_quickDrawable);
+            guiCamera->addChild(qqGeode);
         }
 #endif
-        // Draw first (eg. before Canvas GUI)
-        guiCamera->insertChild(0, geode);
         guiCamera->insertChild(0, FGPanelNode::create2DPanelNode());
     }
 
@@ -677,6 +691,10 @@ FGRenderer::update( ) {
         FGScenerySwitchCallback::scenery_enabled = (sAlpha<1.0);
         _splash_alpha->setDoubleValue((sAlpha < 0) ? 0.0 : sAlpha);
 
+        if (sAlpha <= 0.0) {
+            flightgear::addSentryBreadcrumb("splash-screen fade out complete", "info");
+        }
+
         syncPausePopupState();
         fgSetBool("/sim/menubar/overlap-hide", false);
     }
@@ -745,11 +763,8 @@ FGRenderer::updateSky()
 
     FGLight *l = static_cast<FGLight*>(globals->get_subsystem("lighting"));
 
-    // The sun and moon distances are scaled down versions
-    // of the actual distance to get both the moon and the sun
-    // within the range of the far clip plane.
-    // Moon distance:    384,467 kilometers
-    // Sun distance: 150,000,000 kilometers
+    // The sun and moon distances are scaled down versions of the
+    // actual distance. See FGRenderer::setupView() for more details.
 
     double sun_horiz_eff, moon_horiz_eff;
     if (_horizon_effect->getBoolValue()) {
@@ -805,7 +820,7 @@ FGRenderer::resize( int width, int height )
 
     // update splash node if present
     _splash->resize(width, height);
-#if defined(HAVE_QT)
+#if defined(ENABLE_QQ_UI)
     if (_quickDrawable) {
         _quickDrawable->resize(width, height);
     }
